@@ -263,11 +263,17 @@
   `(list ,text (lambda () ,@cmds)))
 
 (tm-define-macro ($check text check pred?)
-  (:synopsis "Make button")
+  (:synopsis "Make check")
   (if developer-mode?
     (ahash-set! all-translations text #t))
   `(list 'check ,text ,check (lambda () ,pred?)))
 
+(tm-define-macro ($shortcut* text sh)
+  (:synopsis "Make shortcut")
+  (if developer-mode?
+    (ahash-set! all-translations text #t))
+  `(list 'shortcut ,text ,sh))
+
 (tm-define-macro ($balloon text balloon)
   (:synopsis "Make balloon")
   (if developer-mode?
@@ -308,6 +314,10 @@
     (ahash-set! all-translations text #t))
   `(list 'text ,(process-translate text)))
 
+(tm-define-macro ($menu-invisible text)
+  (:synopsis "Make invisible")
+  `(list 'invisible ,text))
+
 (tm-define-macro ($input cmd type proposals width)
   (:synopsis "Make input field")
   `(list 'input (lambda (answer) ,cmd) ,type (lambda () ,proposals) ,width))
