@@ -110,6 +110,8 @@
   (in-text% (and (== (get-env "mode") "text") (not (in-graphics?))))
   (in-math% (and (== (get-env "mode") "math") (not (in-graphics?))))
   (in-prog% (and (== (get-env "mode") "prog") (not (in-graphics?))))
+  (in-hybrid% (inside? 'hybrid))
+  (in-hybrid-math% #t in-hybrid% in-math%)
   (in-math-not-hybrid% (not (inside? 'hybrid)) in-math%)
   (in-math-or-hybrid% (or (in-math?) (inside? 'hybrid)))
   (in-sem-math% (== (get-preference "semantic correctness") "on") in-math%)
@@ -128,17 +130,42 @@
   (in-manual% (not (url-rooted-tmfs? (current-buffer))) in-tmdoc%)
   (in-database% (style-has? "database-style"))
   (in-bib% (style-has? "database-bib-style") in-database%)
+  (in-preview-ref% (style-has? "preview-ref-package"))
+  (in-smart-ref% (style-has? "smart-ref-package"))
   (in-plugin-with-converters%
    (plugin-supports-math-input-ref (get-env "prog-language")))
   (in-screens% (inside? 'screens))
+  (in-article% (style-has? "header-article-package"))
+  (in-book% (style-has? "header-book-package"))
+  (in-letter% (style-has? "header-letter-package"))
+  (in-seminar% (style-has? "header-seminar-package"))
+  (in-generic% (style-has? "generic-style"))
+  (in-browser% (style-has? "browser-style"))
   (in-beamer% (style-has? "beamer-style"))
   (in-poster% (style-has? "poster-style"))
   (in-edu% (style-has? "std-edu-dtd"))
   (in-edu-text% #t in-text% in-edu%)
+  (in-edu-math% #t in-math% in-edu%)
+  (in-auto% (style-has? "automate-dtd"))
+  (in-comment% (style-has? "comment-dtd"))
   (with-any-selection% (selection-active-any?))
   (with-active-selection% (selection-active-normal?))
   (in-cpp% (== (get-env "prog-language") "cpp"))
   (in-prog-cpp% #t in-prog% in-cpp%)
+  (in-dot% (== (get-env "prog-language") "dot"))
+  (in-prog-dot% #t in-prog% in-dot%)
+  (in-octave% (== (get-env "prog-language") "octave"))
+  (in-prog-octave% #t in-prog% in-octave%)
+  (in-java% (== (get-env "prog-language") "java"))
+  (in-prog-java% #t in-prog% in-java%)
+  (in-javascript% (== (get-env "prog-language") "javascript"))
+  (in-prog-javascript% #t in-prog% in-javascript%)
+  (in-json% (== (get-env "prog-language") "json"))
+  (in-prog-json% #t in-prog% in-json%)
+  (in-fortran% (== (get-env "prog-language") "fortran"))
+  (in-prog-fortran% #t in-prog% in-fortran%)
+  (in-scala% (== (get-env "prog-language") "scala"))
+  (in-prog-scala% #t in-prog% in-scala%)
   (in-scheme% (== (get-env "prog-language") "scheme"))
   (in-prog-scheme% #t in-prog% in-scheme%)
   (in-python% (== (get-env "prog-language") "python"))
@@ -197,7 +224,15 @@
   (in-spanish% (== (get-env "language") "spanish") in-text%)
   (in-swedish% (== (get-env "language") "swedish") in-text%)
   (in-taiwanese% (== (get-env "language") "taiwanese") in-oriental%)
-  (in-ukrainian% (== (get-env "language") "ukrainian") in-cyrillic%))
+  (in-ukrainian% (== (get-env "language") "ukrainian") in-cyrillic%)
+
+  (in-math-dutch% (== (get-env "language") "dutch") in-math%)
+  (in-math-english% (== (get-env "language") "english") in-math%)
+  (in-math-french% (== (get-env "language") "french") in-math%)
+  (in-math-german% (== (get-env "language") "german") in-math%)
+  (in-math-italian% (== (get-env "language") "italian") in-math%)
+  (in-math-portuguese% (== (get-env "language") "portuguese") in-math%)
+  (in-math-spanish% (== (get-env "language") "spanish") in-math%))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Keyboard related
@@ -218,6 +253,8 @@
   (like-std% (has-look-and-feel? "std"))
   (simple-menus% (== (get-preference "detailed menus") "simple"))
   (detailed-menus% (== (get-preference "detailed menus") "detailed"))
+  (window-per-buffer% (== (get-preference "buffer management") "separate"))
+  (buffers-share-window% (== (get-preference "buffer management") "share"))
   (with-database-tool% (== (get-preference "database tool") "on"))
   (with-debugging-tool% (== (get-preference "debugging tool") "on"))
   (with-developer-tool% (== (get-preference "developer tool") "on"))
@@ -234,11 +271,10 @@
   (spell-mode% (== (get-input-mode) 3))
   (complete-mode% (== (get-input-mode) 4))
   (remote-control-mode% (== remote-control-flag? #t))
-  (in-cyrillic-cp1251% (cyrillic-input-method? "cp1251") in-cyrillic%)
   (in-cyrillic-jcuken% (cyrillic-input-method? "jcuken") in-cyrillic%)
-  (in-cyrillic-koi8% (cyrillic-input-method? "koi8") in-cyrillic%)
   (in-cyrillic-translit% (cyrillic-input-method? "translit") in-cyrillic%)
-  (in-cyrillic-yawerty% (cyrillic-input-method? "yawerty") in-cyrillic%))
+  (in-cyrillic-yawerty% (cyrillic-input-method? "yawerty") in-cyrillic%)
+  (in-math-like-macos% #t in-math% like-macos%))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Lazy initializations
