@@ -22,6 +22,7 @@
 
 (define cpp-url-last-modified url-last-modified)
 (define cpp-url-newer? url-newer?)
+(define cpp-buffer-last-save buffer-last-save)
 
 (tm-define (url-last-modified u)
   (if (url-rooted-tmfs? u)
@@ -49,6 +50,18 @@
                     (url-test? u "c")))
            (url-glue u suf))))
 
+(tm-define (url-wrap u)
+  (and (url-rooted-tmfs? u)
+       (tmfs-wrap u)))
+
+(tm-define (buffer-last-save u)
+  (with base (url-wrap u)
+    (cond ((not base)
+           (cpp-buffer-last-save u))
+          ((buffer-exists? base)
+           (buffer-last-save base))
+          (else (url-last-modified base)))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Miscellaneous subroutines
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -67,29 +80,39 @@
          (master (if (null? opt-master) (buffer-master) (car opt-master))))
     (aux-set-document aux body)
     (aux-set-master aux master)
-    (switch-to-buffer name)))
+    (switch-document name)))
 
 (define-public-macro (with-aux u . prg)
   `(let* ((u ,u)
           (t (tree-import u "texmacs"))
-          (name (current-buffer)))
-     (open-auxiliary "* Aux *" t u)
+          (name (current-buffer))
+          (aux "* Aux *"))
+     (aux-set-document aux t)
+     (aux-set-master aux u)
+     (switch-to-buffer (aux-name aux))
      (with r (begin ,@prg)
        (switch-to-buffer name)
        r)))
 
 (define buffer-newly-created-table (make-ahash-table))
+(define buffer-initialized-table (make-ahash-table))
 
 (tm-define (buffer-newly-created? name)
   (and name
-       (or (not (buffer-has-name? name))
+       (or (and (not (buffer-has-name? name))
+                (not (ahash-ref buffer-initialized-table name)))
            (ahash-ref buffer-newly-created-table name))))
 
+(tm-define (buffer-initialized name)
+  (ahash-set! buffer-initialized-table name #t))
+
 (tm-define (buffer-copy buf u)
   (:synopsis "Creates a copy of @buf in @u and return @u.")
   (with-buffer buf
     (let* ((styles (get-style-list))
            (init (get-all-inits))
+           (refl (list-references))
+           (refs (map get-reference refl))
            (body (tree-copy (buffer-get-body buf))))
       (view-new u) ; needed by buffer-focus, used in with-buffer
       (buffer-set-body u body) 
@@ -103,13 +126,15 @@
            (if (tree-func? t 'associate)
                (with (var val) (list (tree-ref t 0) (tree-ref t 1))
                  (init-env-tree (tree->string var) val))))
-         (tree-children init)))
+         (tree-children init))
+        (for-each set-reference refl refs))
       u)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Saving buffers
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(tm-define current-save-source (url-none))
 (tm-define current-save-target (url-none))
 
 (define (buffer-notify-recent name)
@@ -127,7 +152,7 @@
 
 (define (save-buffer-save name opts)
   ;;(display* "save-buffer-save " name "\n")
-  (with vname `(verbatim ,(url->system name))
+  (with vname `(verbatim ,(utf8->cork (url->system name)))
     (if (buffer-save name)
         (begin
           (buffer-pretend-modified name)
@@ -162,6 +187,7 @@
 
 (define (save-buffer-check-permissions name opts)
   ;;(display* "save-buffer-check-permissions " name "\n")
+  (set! current-save-source name)
   (set! current-save-target name)
   (with vname `(verbatim ,(url->system name))
     (cond ((url-scratch? name)
@@ -178,8 +204,9 @@
           ((cannot-write? name "Save file")
            (noop))
           ((and (url-test? name "fr")
-                (> (url-last-modified name)
-                   (buffer-last-save name)))
+		(and-with mod-t (url-last-modified name)
+		  (and-with save-t (buffer-last-save name)
+		    (> mod-t save-t))))
            (user-confirm "The file has changed on disk. Really save?" #f
              (lambda (answ)
                (when answ
@@ -272,6 +299,7 @@
 (tm-define (export-buffer-main name to fm opts)
   ;;(display* "export-buffer-main " name ", " to ", " fm "\n")
   (if (string? to) (set! to (url-relative (buffer-get-master name) to)))
+  (if (url? name) (set! current-save-source name))
   (if (url? to) (set! current-save-target to))
   (export-buffer-check-permissions name to fm opts))
 
@@ -367,6 +395,31 @@
 (define-preferences
   ("autosave" "120" notify-autosave))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Opening files using external tools
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (buffer-external? u)
+  (or (and (url-rooted-web? u)
+           ;; FIXME: Use HTTP HEADERS to determine the real file format
+           (!= (file-format u) "texmacs-file"))
+      (file-of-format? u "image")
+      (file-of-format? u "pdf")
+      (file-of-format? u "postscript")
+      (file-of-format? u "generic")))
+
+(tm-define (default-open)
+  (cond ((os-macos?) "open")
+        ((or (os-mingw?) (os-win32?)) "start")
+        (else "xdg-open")))
+
+(tm-define (load-external u)
+  (if (not (url-rooted? u))
+      (set! u (url-relative (current-buffer) u)))
+  (if (url-rooted-web? u)
+      (system (string-append (default-open) " " (url->system u)))
+      (system-1 (default-open) u)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Loading buffers
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -456,11 +509,16 @@
 (tm-define (load-buffer-in-new-window name . opts)
   (:argument name smart-file "File name")
   (:default  name (propose-name-buffer))
-  (apply load-buffer-main (cons name (cons :new-window opts))))
+  (if (buffer->window name)
+      (noop) ;;(window-focus (buffer->window name))
+      (apply load-buffer-main (cons name (cons :new-window opts)))))
 
 (tm-define (load-browse-buffer name)
   (:synopsis "Load a buffer or switch to it if already open")
-  (if (buffer-exists? name) (switch-to-buffer name) (load-buffer name)))
+  (cond ((buffer-exists? name) (switch-to-buffer name))
+        ((url-rooted-web? (current-buffer)) (load-buffer name))
+        ((buffer-external? name) (load-external name))
+        (else (load-buffer name))))
 
 (tm-define (open-buffer)
   (:synopsis "Open a new file")
@@ -520,24 +578,88 @@
   (lambda (s) (import-buffer s fm)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Printing buffers
+;; System dependent conventions for buffer management
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(tm-define (print-buffer)
-  (:synopsis "Print the current buffer")
-  (print))
+(tm-define (open-in-window)
+  (choose-file load-buffer-in-new-window "Load file" ""))
+
+(tm-define (open-document)
+  (if (window-per-buffer?) (open-in-window) (open-buffer)))
+
+(tm-define (open-document*)
+  (if (window-per-buffer?) (open-buffer) (open-in-window)))
+
+(tm-define (load-document u)
+  (:argument u smart-file "File name")
+  (:default  u (propose-name-buffer))
+  (when (not (url-none? u))
+    (if (window-per-buffer?) (load-buffer-in-new-window u) (load-buffer u))))
+
+(tm-define (load-document* u)
+  (:argument u smart-file "File name")
+  (:default  u (propose-name-buffer))
+  (when (not (url-none? u))
+    (if (window-per-buffer?) (load-buffer u) (load-buffer-in-new-window u))))
+
+(tm-define (switch-document u)
+  (:argument u smart-file "File name")
+  (:default  u (propose-name-buffer))
+  (when (not (url-none? u))
+    (if (window-per-buffer?)
+        (if (buffer->window u)
+            (noop) ;;(window-focus (buffer->window u))
+            (open-buffer-in-window u (buffer-get u) ""))
+        (load-buffer u))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Printing buffers
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (tm-define (interactive-page-setup)
   (:synopsis "Specify the page setup")
   (:interactive #t)
   (set-message "Not yet implemented" "Printer setup"))
 
+(tm-define (direct-print-buffer)
+  (:synopsis "Print the current buffer")
+  (print))
+
 (tm-define (interactive-print-buffer)
   (:synopsis "Print the current buffer")
   (:interactive #t)
   (print-to-file "$TEXMACS_HOME_PATH/system/tmp/tmpprint.ps")
   (interactive-print '() "$TEXMACS_HOME_PATH/system/tmp/tmpprint.ps"))
 
+(tm-define (print-buffer)
+  (:synopsis "Print the current buffer")
+  (:interactive (use-print-dialog?))
+  (if (use-print-dialog?)
+      (interactive-print-buffer)
+      (direct-print-buffer)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Important files to which the buffer is linked (e.g. bibliographies)
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define (linked-files-inside t)
+  (cond ((tree-atomic? t) (list))
+        ((tree-is? t 'document)
+         (append-map linked-files-inside (tree-children t)))
+        ((tree-in? t '(with with-bib))
+         (linked-files-inside (tm-ref t :last)))
+        ((or (tree-func? t 'bibliography 4)
+	     (tree-func? t 'bibliography* 5))
+         (with name (tm->stree (tm-ref t 2))
+           (if (or (== name "") (nstring? name)) (list)
+               (with s (if (string-ends? name ".bib") name
+                           (string-append name ".bib"))
+                 (list (url-relative (current-buffer) s))))))
+        (else (list))))
+
+(tm-define (linked-file-list)
+  (linked-files-inside (buffer-tree)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Deprecated functionality
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
