@@ -16,17 +16,6 @@
 	(utils edit auto-close)
 	(text text-edit)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Hacks to work around the problem that MacOS reserves Alt-based shortcuts
-;; for its own personal use
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-
-(when (os-macos?)
-  (kbd-map
-    (:mode in-std-text?)
-    ("\x0f" (make-equation*)) ;; Alt-$
-    ("ddagger" (make-eqnarray*)))) ;; Alt-&
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Special symbols in text mode
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -271,31 +260,66 @@
   ("accent:ogonek e" "")
   ("accent:ogonek E" "")
 
+  ("" "")
+  ("" "")
+  ("" "")
+  ("" "")
+  ("" "")
+  ("" "")
+  ("" "")
+  ("" "")
+  ("" "")
+
   ("exclamdown" "")
-  ("cent" (make 'cent))
+  ("cent" "<#A2>")
   ("sterling" "")
-  ("currency" (make 'currency))
-  ("yen" (make 'yen))
+  ("currency" "<#A4>")
+  ("yen" "<#A5>")
   ("section" "")
-  ("copyright" (make 'copyright))
+  ("copyright" "<#A9>")
   ("copyright var" (make 'copyleft))
+  ("ordfeminine" "<#AA>")
+  ("ordfeminine var" (make 'varordfeminine))
   ("guillemotleft" "")
-  ("registered" (make 'registered))
-  ("degree" (make 'degreesign))
-  ("twosuperior" (make 'twosuperior))
-  ("threesuperior" (make 'threesuperior))
-  ("paragraph" (make 'paragraphsign))
-  ("onesuperior" (make 'onesuperior))
-  ("guillemotright" "")
-  ("onequarter" (make 'onequarter))
-  ("onehalf" (make 'onehalf))
-  ("threequarters" (make 'threequarters))
-  ("questiondown" "")
-  ("euro" (make 'euro))
-  ("masculine" (make 'masculine))
-  ("ordfeminine" (make 'ordfeminine))
+  ("registered" "<#AE>")
+  ("circledR" "<#AE>")
+  ("degree" "<#B0>")
+  ("twosuperior" "<#B2>")
+  ("threesuperior" "<#B3>")
+  ("paragraph" "<#B6>")
+  ("onesuperior" "<#B9>")
+  ("masculine" "<#BA>")
   ("masculine var" (make 'varmasculine))
-  ("ordfeminine var" (make 'varordfeminine)))
+  ("guillemotright" "")
+  ("onequarter" "<#BC>")
+  ("onehalf" "<#BD>")
+  ("threequarters" "<#BE>")
+  ("questiondown" "")
+
+  ("trademark" "<#2122>")
+  ("big-sum" (insert '(big "sum")))
+  ("dagger" "<dag>")
+  ("sqrt" (insert '(sqrt "")))
+  ("big-int" (insert '(big "int")))
+  ("euro" "<#20AC>")
+  ("ddagger" "<ddag>")
+  ("centerdot" "<cdot>")
+  ("big-prod" (insert '(big "prod")))
+  ("varspace" (insert '(nbsp))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Font dependent shortcuts
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(kbd-map
+  (:mode in-text?)
+  (:require (== (get-env "font") "roman"))
+
+  ("copyright" (make 'copyright)) ;; "<#A9>"
+  ("ordfeminine" (make 'ordfeminine))
+  ("masculine" (make 'masculine))
+  ("<#20AC>" (make 'euro))
+  ("euro" (make 'euro)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Language dependent shortcuts
